@@ -11,6 +11,8 @@ from pandas.core.series import Series
 from bender.data_importer.importer import AppendImporter, CachedImporter, DataImportable, DataImporter, JoinedImporter
 from bender.evaluator.factory_method import Evaluable
 from bender.evaluator.interface import Evaluator
+from bender.explorer.factory import Explorable
+from bender.explorer.interface import Explorer
 from bender.metric.factory import Metricable
 from bender.metric.interface import Metric
 from bender.model_exporter.factory import ModelExportable
@@ -20,12 +22,31 @@ from bender.model_trainer.interface import ModelTrainer, Trainable
 from bender.pipeline.interface import RunnablePipeline
 from bender.prediction_extractor import Predictable, PredictionExtractor, PredictionOutput
 from bender.split_strategy.split_strategy import Splitable, SplitStrategy, TrainingDataSet
-from bender.trained_model.interface import TrainedModel
+from bender.trained_model.interface import TrainedModel, TrainedProbabilisticModel
 from bender.transformation.transformation import Processable, Transformation
 
 logger = logging.getLogger(__name__)
 
 
+class ExplorePipeline(RunnablePipeline[DataFrame], Splitable):
+
+    pipeline: RunnablePipeline[DataFrame]
+    explorers: list[Explorer]
+
+    def __init__(self, pipeline: RunnablePipeline[DataFrame], explorers: list[Explorer]) -> None:
+        self.pipeline = pipeline
+        self.explorers = explorers
+
+    async def run(self) -> DataFrame:
+        df = await self.pipeline.run()
+        for explorer in self.explorers:
+            await explorer.explore(df)
+        return df
+
+    def split(self, split_strategy: SplitStrategy) -> SplitedData:
+        return SplitedData(self, split_strategy)
+
+
 class LoadedDataAndModel(
     Processable,
     Predictable,
@@ -46,6 +67,9 @@ class LoadedDataAndModel(
     def predict(self, on: Optional[Callable[[DataFrame], Series]] = None) -> PredictionPipeline:
         return PredictionPipeline(self, predict_on=on)
 
+    def predict_proba(self, on: Optional[Callable[[DataFrame], Series]] = None) -> ProbalisticPredictionPipeline:
+        return ProbalisticPredictionPipeline(self, predict_on=on)
+
     async def run(self) -> tuple[TrainedModel, DataFrame]:
         processed_data = await self.data_loader.run()
         loaded_model = await self.model_loader.run()
@@ -73,6 +97,31 @@ class PredictionPipeline(RunnablePipeline[Series]):
         return model.predict(predict_on_data)
 
 
+class ProbalisticPredictionPipeline(RunnablePipeline[DataFrame]):
+
+    data_and_model: RunnablePipeline[tuple[TrainedProbabilisticModel, DataFrame]]
+    predict_on: Optional[Callable[[DataFrame], Series]]
+
+    def __init__(
+        self,
+        data_and_model: RunnablePipeline[tuple[TrainedProbabilisticModel, DataFrame]],
+        predict_on: Optional[Callable[[DataFrame], Series]],
+    ) -> None:
+        self.data_and_model = data_and_model
+        self.predict_on = predict_on
+
+    async def run(self) -> DataFrame:
+        model, processed_data = await self.data_and_model.run()
+
+        if self.predict_on:
+            predict_on_filter = self.predict_on(processed_data)
+            predict_on_data = processed_data[predict_on_filter]
+        else:
+            predict_on_data = processed_data
+
+        return model.predict_proba(predict_on_data)
+
+
 class ExtractFromPredictionPipeline(RunnablePipeline[None]):
 
     data_and_model: LoadedDataAndModel
@@ -137,6 +186,7 @@ class LoadedData(
     RunnablePipeline[DataFrame],
     Splitable,
     DataImporter,
+    Explorable[ExplorePipeline],
 ):
 
     importer: DataImporter
@@ -202,13 +252,16 @@ class LoadedData(
         """
         return LoadedData(AppendImporter(self.importer, importer), self.transformations)
 
+    def explore(self, explorers: list[Explorer]) -> ExplorePipeline:
+        return ExplorePipeline(self, explorers)
 
-class SplitedData(RunnablePipeline[tuple[DataFrame, DataFrame]], Trainable['TrainingPipeline']):
 
-    data_loader: LoadedData
+class SplitedData(RunnablePipeline[tuple[DataFrame, DataFrame]], Trainable):
+
+    data_loader: RunnablePipeline[DataFrame]
     split_strategy: SplitStrategy
 
-    def __init__(self, data_loader: LoadedData, split_strategy: SplitStrategy) -> None:
+    def __init__(self, data_loader: RunnablePipeline[DataFrame], split_strategy: SplitStrategy) -> None:
         self.data_loader = data_loader
         self.split_strategy = split_strategy
 
@@ -292,6 +345,7 @@ class TrainAndEvaluatePipeline(
     async def run(self) -> tuple[TrainedModel, TrainingDataSet]:
         model, train_set = await self.pipeline.run()
         for evaluator in self.evaluators:
+            logger.info(f'Evaluating with evaloator: {type(evaluator)}')
             await evaluator.evaluate(model, train_set)
         return model, train_set
 
